@@ -1,21 +1,22 @@
 #-*- coding: utf-8 -*-
 
-#    This program is free software; you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 3 of the License, or
-#    (at your option) any later version.
-#
-#    This program is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License along
-#    with this program; if not, write to the Free Software Foundation, Inc.,
-#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-#
-# (C) 2012 - Pierre-Yves Chibon <pingou@pingoured.fr>
+"""
+ This program is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; either version 3 of the License, or
+ (at your option) any later version.
 
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
+ (C) 2012 - Pierre-Yves Chibon <pingou@pingoured.fr>
+"""
 
 import argparse
 import ConfigParser
@@ -31,7 +32,7 @@ from tarfile import TarError
 logging.basicConfig()
 LOG = logging.getLogger('Pypi2spec')
 #LOG.setLevel('DEBUG')
-__version__ = '0.2.2'
+__version__ = '0.3.0'
 
 
 def create_conf(configfile):
@@ -87,12 +88,35 @@ def move_sources(fullpath, sources):
     shutil.copyfile(fullpath, dest)
 
 
+def get_packager_name():
+    """ Query rpm to retrieve a potential packager name from the 
+    .rpmmacros.
+    """
+    packager = get_rpm_tag('%packager')
+    if not packager.startswith('%'):
+        packager = packager.split('<')[0].strip()
+        return packager
+    else:
+        return ''
+
+def get_packager_email():
+    """ Query rpm to retrieve a potential packager email from the 
+    .rpmmacros.
+    """
+    packager = get_rpm_tag('%packager')
+    if not packager.startswith('%'):
+        packager = packager.split('<', 1)[1].rsplit('>', 1)[0].strip()
+        return packager
+    else:
+        return ''
+
+
 class Settings(object):
     """ Pypi2spec user config Setting"""
     # Editor to use in the spec
-    packager = os.getlogin()
+    packager = get_packager_name()
     # Editor email to use in the spec
-    email = ''
+    email = get_packager_email()
 
     def __init__(self):
         """Constructor of the Settings object.
@@ -166,13 +190,14 @@ class Pypi2specError(Exception):
     Template for all the error of the project
     """
 
-    def __init__(self, value):
+    def __init__(self, message):
         """ Constructor. """
-        self.value = value
+        super(Pypi2specError, self).__init__(message)
+        self.message = message
 
     def __str__(self):
         """ Represent the error. """
-        return str(self.value)
+        return str(self.message)
 
 
 class Pypi2spec(object):
@@ -185,6 +210,7 @@ class Pypi2spec(object):
         :arg name, the name of the library on the pypi website.
         """
         self.name = name
+        self.description = ''
         self.log = get_logger()
         self.version = ''
         self.summary = ''
@@ -240,7 +266,11 @@ class Pypi2spec(object):
         url = '{url}/{source}'.format(url=url, source=self.source)
         self.log.info('Downloading %s' % url)
 
-        remotefile = urllib2.urlopen(url)
+        try:
+            remotefile = urllib2.urlopen(url)
+        except urllib2.HTTPError, err:
+            self.log.debug(err)
+            raise Pypi2specError('Could not retrieve source: %s' % url)
         localfile = open(sources, 'w')
         localfile.write(remotefile.read())
         localfile.close()
@@ -276,41 +306,66 @@ class Pypi2spec(object):
         file.
         """
         import rdflib
-        g = rdflib.Graph()
+        graph = rdflib.Graph()
         try:
-            g.parse('http://pypi.python.org/pypi?:action=doap&name=%s' % \
+            graph.parse('http://pypi.python.org/pypi?:action=doap&name=%s' % \
                 self.name, format='xml')
         except urllib2.HTTPError, err:
             self.log.debug('ERROR while downloading the doap file:\n  %s'
                 % err)
             raise Pypi2specError('Could not retrieve information for the'
                 'project "%s". Did you make a typo?' % self.name)
-        DOAP = rdflib.Namespace('http://usefulinc.com/ns/doap#')
-        RDFS = rdflib.Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#')
-        version_node = g.value(predicate=RDFS['type'],
-            object=DOAP['Version'])
-        self.version = g.value(subject=version_node,
-            predicate=DOAP['revision'])
-        project_node = g.value(predicate=RDFS['type'],
-            object=DOAP['Project'])
-        self.summary = g.value(subject=project_node,
-            predicate=DOAP['shortdesc'])
-        self.description = g.value(subject=project_node,
-            predicate=DOAP['description'])
-        self.source0 = g.value(subject=project_node,
-            predicate=DOAP['download-page'])
+        doap = rdflib.Namespace('http://usefulinc.com/ns/doap#')
+        rdfs = rdflib.Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#')
+        version_node = graph.value(predicate=rdfs['type'],
+            object=doap['Version'])
+        self.version = graph.value(subject=version_node,
+            predicate=doap['revision'])
+        project_node = graph.value(predicate=rdfs['type'],
+            object=doap['Project'])
+        self.summary = graph.value(subject=project_node,
+            predicate=doap['shortdesc'])
+        self.description = graph.value(subject=project_node,
+            predicate=doap['description'])
+        self.source0 = graph.value(subject=project_node,
+            predicate=doap['download-page'])
 
-        if not self.source0:
+        if not self.source0 \
+            or os.path.splitext(self.source0)[1] not in \
+            ['.gz', '.zip', '.bz2']:
             pypi_base = 'http://pypi.python.org/packages/source/'
-            self.source0 = pypi_base + '%s/%s/%s-%s.tar.gz' % \
+            self.source0 = pypi_base + '%s/%s/%s-%s' % \
                 (self.name[0], self.name, self.name, self.version)
 
-        self.source = '%s-%s.tar.gz' % (self.name, self.version)
+            url_ext = False
+            for ext in ['tar.gz', 'zip', 'tar.bz2']:
+                url = '%s.%s' % (self.source0, ext)
+                self.log.debug(url)
+                try:
+                    urllib2.urlopen(url)
+                    url_ext = ext
+                except urllib2.HTTPError, err:
+                    self.log.debug(err)
+            if url_ext is not False:
+                self.source0 = '%s.%s' % (self.source0, url_ext)
+                self.source = '%s-%s.%s' % (self.name, self.version,
+                    url_ext)
+            else:
+                raise Pypi2specError('No tarball or zip file could be '
+                    'found for this package: %s' % self.name)
+        else:
+            self.source = '%s-%s.tar.gz' % (self.name, self.version)
 
 
 class Pypi2specUI(object):
     """ Class handling the user interface. """
 
+    def __init__(self):
+        """ Constructor.
+        """
+        self.parser = None
+        self.log = get_logger()
+
     def setup_parser(self):
         """ Command line parser. """
         self.parser = argparse.ArgumentParser(usage='%(prog)s [options]',
@@ -319,6 +374,8 @@ class Pypi2specUI(object):
             version='%(prog)s ' + __version__)
         self.parser.add_argument('package',
             help='Name of the pypi library to package.')
+        self.parser.add_argument('--python3', action='store_true',
+            help='Create a specfile for both python2 and python3.')
         self.parser.add_argument('--verbose', action='store_true',
             help='Give more info about what is going on.')
         self.parser.add_argument('--debug', action='store_true',
@@ -346,7 +403,7 @@ class Pypi2specUI(object):
             pypi.determine_arch()
             pypi.remove_sources()
             settings = Settings()
-            spec = Spec(settings, pypi)
+            spec = Spec(settings, pypi, python3=args.python3)
             spec.fill_spec_info()
             spec.get_template()
             spec.write_spec()
@@ -355,7 +412,12 @@ class Pypi2specUI(object):
 
 
 def main():
+    """ Entry point used in the setup.py.
+    This just calls the main function in Pypi2specUI.
+    """
     return Pypi2specUI().main()
 
 if __name__ == '__main__':
+    #import sys
+    #sys.argv.append('pypi2spec')
     main()
